/*-
 * %%----------------------------------------------------------------------------------------------
 * Solidify Framework - Solidify Controller - JoinResourceNTiersService.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.unige.solidify.service;

import org.springframework.beans.factory.annotation.Autowired;

import ch.unige.solidify.repository.JoinRepository;
import ch.unige.solidify.rest.JoinResource;
import ch.unige.solidify.rest.Resource;

public abstract class JoinResourceNTiersService<T extends Resource, V extends Resource, J extends JoinResource<?>> extends
        AbstractResourceService {

  @Autowired
  protected JoinRepository<J> joinRepository;

  /****************************************************************************************************/

  /**
   * Save the relation
   *
   * @param joinResource
   * @return
   */
  public J saveRelation(J joinResource) {
    this.setAuthenticatedUserProperties(joinResource);
    return this.joinRepository.save(joinResource);
  }

  /**
   * Delete the relation
   *
   * @param joinResource
   */
  public void deleteRelation(J joinResource) {
    this.joinRepository.delete(joinResource);
  }

  /**
   * Return a JoinResource with eventual default relation's properties
   * <p>
   * By default return an join resource with no additional property. Override to set them if there are any.
   *
   * @return
   */
  public J getDefaultJoinResource() {
    return this.getEmptyJoinResourceObject();
  }

  /****************************************************************************************************/

  /**
   * Return a new instance of join resource
   *
   * @return
   */
  public abstract J getEmptyJoinResourceObject();

  /**
   * Return a new instance of parent
   *
   * @return
   */
  public abstract T getEmptyParentResourceObject();

  /**
   * Return a new instance of child
   *
   * @return
   */
  public abstract V getEmptyChildResourceObject();

  /**
   * Set the parent property of the join resource
   *
   * @param joinResource
   * @param parentResource
   */
  public abstract void setParentResource(J joinResource, T parentResource);

  /**
   * Set the child property of the join resource
   *
   * @param joinResource
   * @param childResource
   */
  public abstract void setChildResource(J joinResource, V childResource);
}
